//
//  ContentView.swift
//  MyBirthdayTimer
//
//  Created by 大津 真 on 2020/04/14.
//  Copyright © 2020 大津 真. All rights reserved.
//

import SwiftUI

// 保存用のキー
let birthKey = "myDate"

struct ContentView: View {
    // 誕生日の日付
    @State private var birthDate = Date()
    // 誕生日が保存されているかどうか
    @State private var isSaved = false
    
    // タイマーを使って3秒毎に残り日数を計算して画面を更新する
    @State var timer: Timer?
    @State private var time = Date()
    @State private var onOf = true
    @State private var daysLeft = 0
    @State private var age = 0

    var body: some View {
        NavigationView {
            VStack {
                NavigationLink(destination: SetBirthdateView(birthDate: $birthDate, isSaved: $isSaved)) {
                    if !isSaved {
                        Text("誕生日を設定")
                    } else {
                        Text("誕生日: \(jp_date(birthDate))")
                    }
                }
                .navigationBarTitle("誕生日リマインダー")
                
                if isSaved {
                    // タイマーの動作を確認するために日時を表示する
                    Text("\(time)")
                        .font(.caption)

                    // タイマーの動作をわかりやすくするために3秒毎に文字色を変更する
                    if daysLeft == 0 {
                        Text("ハッピーバースデイ")
                            .foregroundColor(onOf ? .orange : . yellow)
                        Text("年齢\(self.age)才")
                            .font(.largeTitle)
                            .padding()
                    } else {
                        Text("誕生日まであと\(daysLeft)日")
                            .foregroundColor(onOf ? .black : . gray)
                        Text("年齢\(self.age)才")
                            .font(.largeTitle)
                            .padding()
                    }
                }
            }
            .font(.title)
        }
        .onAppear{
            // 誕生日を読み込む
            if let birthDate  = UserDefaults.standard.object(forKey: birthKey) {
                self.birthDate = birthDate as! Date
                self.isSaved = true
            }
            self.daysLeft = self.calcDaysLeft()
            self.age = self.calcAge()
            // タイマーを起動して3秒毎に日付を計算する
            self.timer = Timer.scheduledTimer(withTimeInterval: 3, repeats: true){_ in
                self.time = Date()
                self.daysLeft = self.calcDaysLeft()
                self.age = self.calcAge()
                self.onOf.toggle()
            }
        }
    }
    
    // 年齢を返す
    func calcAge() -> Int {
        let cal = Calendar(identifier: .gregorian)
        let now = Date()
        return cal.dateComponents([.year], from: birthDate, to: now).year!
    }

    // 誕生日までの残り日数を戻す
    func calcDaysLeft() -> Int {
        let cal = Calendar(identifier: .gregorian)
        let now = Date()
        var comp = cal.dateComponents([.year, .month, .day,], from: now)
        // 今日の0時0分のDateオブジェクトをtodayに代入
        let today = cal.date(from: comp)!
        // nextBirthDateを今年の誕生日に設定
        let thisYear = cal.component(.year, from: now)
        comp = cal.dateComponents([.year, .month, .day], from: self.birthDate)
        comp.year = thisYear
        var nextBirthDate = cal.date(from: comp)!
        // 誕生日が過ぎていたらnextBirthDateを来年の誕生日に設定
        if nextBirthDate < today {
            comp = cal.dateComponents([.year, .month, .day,], from: nextBirthDate)
            comp.year! += 1
            nextBirthDate = cal.date(from: comp)!
        }
        //　誕生日までの日数を計算する
        return cal.dateComponents([.day], from: today, to: nextBirthDate).day!
    }
    
    // 日本語の日付を取得
    func jp_date(_ date: Date) -> String {
        let df = DateFormatter()
        df.locale = Locale(identifier: "ja_JP")
        df.dateStyle = .full
        return df.string(from: date)
    }
}

struct ContentView_Previews: PreviewProvider {
    static var previews: some View {
        ContentView()
    }
}
